/**
	Haed a free program maker and generator
	Copyright (C) 2018  Florian Evaldsson

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/
#include "includes.h"

/*
	Ta bort delarna som inte anger object i horisontellt led?
*/

/**
	Add the action stuff to the new button depending on object

	@param button
		button to change
	@param condition
		the condition for later usage
	@param object
		the object, with its type
*/
void haed_triggers_bind_object_to_button(GtkWidget *button,EksParent *condition,EksParent *object)
{
	if(strcmp(object->upperEksParent->name,ACTIVE_OBJECT_S)==0)
	{
		g_signal_connect(button, "button_press_event",G_CALLBACK(OnAOButtonActionSelect),NULL);
	}
	else if(strcmp(object->upperEksParent->name,TRIGGERS_SPECIAL_S)==0)
	{
		g_signal_connect(button, "button_press_event",G_CALLBACK(haed_trigger_actions_special),NULL);
	}
	else if(strcmp(object->upperEksParent->name,TRIGGERS_STORY_S)==0)
	{
		g_signal_connect(button, "button_press_event",G_CALLBACK(haed_trigger_actions_storyboard),NULL);
	}
	
	g_object_set_data(G_OBJECT(button),"haed_object",(void*)object);
	g_object_set_data(G_OBJECT(button),"haed_condition",(void*)condition);
}

/**
	Adds an object to the Trigger Handler.
	Also adds buttons for the new object into the trigger fields. otherwise, it would be impossible to set triggers for new objects.

	@param object NO_FREE
		the object to add.
	@param objectType NO_FREE
		the string of the object type.
	@return .
		void
*/
void haed_trigghandler_add_object(EksParent *object)
{
	TriggerHandler *triggHandler=HAED_GLOBAL_TRIGGER_HANDLER;

	triggHandler->columnAmount++;
	
	for(int i=0;i<triggHandler->rowAmount;i++)
	{
		
		//resize the size of all row buttons. (dynamic usage)
		triggHandler->row[i]->ActionButtons=realloc(triggHandler->row[i]->ActionButtons,sizeof(GtkWidget*)*(triggHandler->columnAmount));
		
		triggHandler->row[i]->ActionButtons[triggHandler->columnAmount-1]=gtk_button_new_with_label(" ");
		gtk_grid_attach(GTK_GRID(HAED_GLOBAL_TRIGGER_GRID),triggHandler->row[i]->ActionButtons[triggHandler->columnAmount-1],3+triggHandler->columnAmount-1,2+i,1,1);
		gtk_widget_show(triggHandler->row[i]->ActionButtons[triggHandler->columnAmount-1]);
 
		haed_triggers_bind_object_to_button(triggHandler->row[i]->ActionButtons[triggHandler->columnAmount-1],triggHandler->row[i]->toConditionLinker,object);
	}
	
	//resize the object structure
	triggHandler->objects=realloc(triggHandler->objects,sizeof(EksParent*)*(triggHandler->columnAmount));
	
	triggHandler->objects[triggHandler->columnAmount-1]=object;
}

/**
	Free the triggerhandler struct, used in new-program for example, or if destroying a map.
	Wont free any internal widgets, only freeing the pointers to them
	
	@param triggHandler
		the trigghandler to free
*/
void haed_trigghandler_clean(void)
{
	TriggerHandler *triggHandler=HAED_GLOBAL_TRIGGER_HANDLER;

	if(triggHandler)
	{
		for(int i=0;i<triggHandler->rowAmount;i++)
		{
			UITriggRow *thisRow=triggHandler->row[i];
		
			free(thisRow->ActionButtons);
			free(thisRow);
		}
	
		free(triggHandler->row);
		free(triggHandler->objects);
		
		memset(triggHandler,0,sizeof(TriggerHandler));
	}
}

/**
	Counts the number of objects in that area.

	@param triggHandler NO_FREE
		The trigger handler to search in
	@param objectType NO_FREE
		name of the type.
	@return .
		returns the amount.
*/
int haed_trigghandler_count_objects(const char *objectType)
{
	TriggerHandler *triggHandler=HAED_GLOBAL_TRIGGER_HANDLER;
	//b_tm("ERP? %d",triggHandler->columnAmount);
	int amount=0;
	
	for(int i=0;i<triggHandler->columnAmount;i++)
	{
		//b_tm("TRIGG MATCH %s %s %s",triggHandler->objects[i]->upperEksParent->name,triggHandler->objects[i]->name,objectType);
		
		if(triggHandler->objects[i] && strcmp(triggHandler->objects[i]->upperEksParent->name,objectType)==0)
		{
			amount++;
		}
	}
	
	return amount;
}

/**
	Get an object from a position.
	
	@param triggHandler NO_FREE
		The trigger handler to search in
	@param objectType NO_FREE
		name of the type.
	@param pos .
		position of the object
	@return POINTER
		the object
		or NULL if it failed.
*/

EksParent *haed_trigghandler_get_object(const char *objectType,int pos)
{
	TriggerHandler *triggHandler=HAED_GLOBAL_TRIGGER_HANDLER;

	int amount=0;

	for(int i=0;i<triggHandler->columnAmount;i++)
	{
		if(triggHandler->objects[i] && strcmp(triggHandler->objects[i]->upperEksParent->name,objectType)==0)
		{
			if(amount==pos)
			{
				return triggHandler->objects[i];
			}
			amount++;
		}
	}
	
	//get nothing
	return NULL;
}

EksParent *haed_triggerhandler_get_object_by_name(char *name)
{
	TriggerHandler *triggHandler=HAED_GLOBAL_TRIGGER_HANDLER;

	for(int i=0;i<triggHandler->columnAmount;i++)
	{
		if(triggHandler->objects[i] && strcmp(triggHandler->objects[i]->name,name)==0)
		{
			return triggHandler->objects[i];
		}
	}

	return NULL;
}

/**
	Remove condition.
	

*/
void haed_trigghandler_remove_condition(GtkWidget *triggerGrid,EksParent *conditionIn)
{
	TriggerHandler *handler=HAED_GLOBAL_TRIGGER_HANDLER;

	UITriggRow *currentRow;
	
	/*	EksParent *returnObject=NULL;*/
	/*	*/
	/*	GList *firstContain=gtk_container_get_children(GTK_CONTAINER(gtk_container_get_children(GTK_CONTAINER(HAED_GLOBAL_TRIGGER_GRID))->data));*/
	/*	GList *firstChild=firstContain;//gtk_container_get_children(GTK_CONTAINER(HAED_GLOBAL_TRIGGER_GRID));*/
	/*	GList *currentChild=firstChild;*/
	/*	*/
	/*	do*/
	/*	{*/
	/*		EksParent *tempObject=g_object_get_data(G_OBJECT(currentChild->data),"haed_object");*/

	/*		if(tempObject && strcmp(tempObject->type,objectType)==0)*/
	/*		{*/
	/*			if(amount==pos)*/
	/*			{*/
	/*				returnObject=tempObject;*/
	/*			}*/
	/*			amount++;*/
	/*		}*/
	/*		*/
	/*		currentChild=currentChild->next;*/
	/*	*/
	/*	}while(currentChild!=firstChild && currentChild!=NULL);*/

/*	return returnObject;*/
	
	for(int i=0;i<(handler->rowAmount);i++)
	{
		currentRow=handler->row[i];
		
		if(currentRow->toConditionLinker==conditionIn)
		{
			for(int ii=0;ii<(handler->columnAmount);ii++)
			{
				gtk_widget_destroy(currentRow->ActionButtons[ii]);
			}
			
			gtk_widget_destroy(currentRow->numberInf);
			gtk_widget_destroy(currentRow->conditionInf);
			
			eks_parent_destroy(conditionIn,EKS_TRUE);
			
			free(currentRow->ActionButtons);
			free(currentRow);
			//currentRow=NULL;
		}
	}
}

int haed_triggerhandler_get_pos_from_object(EksParent *object)
{
	TriggerHandler *triggHandler=HAED_GLOBAL_TRIGGER_HANDLER;

	for(int i=0;i<triggHandler->columnAmount;i++)
	{
		if(triggHandler->objects[i]==object)
		{
			return i;
		}
	}
	
	return 0;
}

